---------------
-- Variables --
---------------
Perl_Player_Buff_Config = {};

-- Default Saved Variables (also set in Perl_Player_Buff_GetVars)
local buffalerts = 1;		-- alerts are enabled by default
local showbuffs = 1;		-- mod is on be default
local scale = 1;		-- default scale

-- Default Local Variables
local BuffAlphaMax = 1;		-- Max alpha.
local BuffAlphaMin = 0.25;	-- Minimum alpha for the sine modulation.
local BuffCycleSpeed = 0.75;	-- Cycles per second.
local BuffWarnTime = 31;	-- Onset of flashing
local Initialized = nil;	-- waiting to be initialized
local transparency = 1;		-- 0.8 default from perl
local debuffcheck = {		-- table used for debuff stuff below
	[0] = 0,
	[1] = 0,
	[2] = 0,
	[3] = 0,
	[4] = 0,
	[5] = 0,
	[6] = 0,
	[7] = 0,
	[8] = 0,
	[9] = 0,
	[10] = 0,
	[11] = 0,
	[12] = 0,
	[13] = 0,
	[14] = 0,
	[15] = 0,
	[16] = 0,
	[17] = 0,
	[18] = 0,
	[19] = 0,
	[20] = 0,
	[21] = 0,
	[22] = 0,
	[23] = 0,
	[24] = 0,
}


----------------------
-- Loading Function --
----------------------
function Perl_Player_Buff_OnLoad()
	-- Events
	this:RegisterEvent("PLAYER_ENTERING_WORLD");
	this:RegisterEvent("VARIABLES_LOADED");

	if (DEFAULT_CHAT_FRAME) then
		DEFAULT_CHAT_FRAME:AddMessage("|cffffff00Perl Classic: Player_Buff loaded successfully.");
	end
end

function Perl_Player_Buff_Buff_OnLoad()
	this:RegisterForClicks("RightButtonUp");
end


---------------------------
-- Event/Update Handlers --
---------------------------
function Perl_Player_Buff_OnEvent(event)
	if (event=="PLAYER_ENTERING_WORLD" or event == "VARIABLES_LOADED") then
		Perl_Player_Buff_Initialize();
	else
		return;
	end
end


-------------------------------
-- Loading Settings Function --
-------------------------------
function Perl_Player_Buff_Initialize()
	-- Code to be run after zoning or logging in goes here
	if (Initialized) then
		Perl_Player_Buff_Set_Scale();
		return;
	end

	-- Check if a previous exists, if not, enable by default.
	if (type(Perl_Player_Buff_Config[UnitName("player")]) == "table") then
		Perl_Player_Buff_GetVars();
	else
		Perl_Player_Buff_UpdateVars();
	end

	-- Major config options.
	Perl_Player_BuffFrame:SetBackdropColor(0, 0, 0, 1);
	Perl_Player_BuffFrame:SetBackdropBorderColor(0.5, 0.5, 0.5, 1);
	Perl_Player_UseBuffs(showbuffs);

	-- MyAddOns Support
	Perl_Player_Buff_myAddOns_Support();

	Initialized = 1;
end


--------------------
-- Buff Functions --
--------------------
function Perl_Player_Buff_UpdateAll()
	if (UnitName("player") and (showbuffs == 1)) then
		Perl_Player_BuffFrame:Show();

		for buffnum=1,24 do
			local button = getglobal("Perl_Player_Buff"..buffnum);
			local buffIndex, untilCancelled = GetPlayerBuff((buffnum - 1), "HELPFUL|HARMFUL");

			if (buffIndex > -1) then
				local icon = getglobal(button:GetName().."Icon");
				local debuff = getglobal(button:GetName().."DebuffBorder");
				local timetext = getglobal(button:GetName().."DurationText");
				local buffname = Perl_Player_GetBuffName(buffIndex);
				local debuffCount = getglobal(button:GetName().."Count");

				icon:SetTexture(GetPlayerBuffTexture(buffIndex));

				local debuffApplications = GetPlayerBuffApplications(buffIndex);
				if (debuffApplications > 1) then
					debuffCount:SetText(debuffApplications);
					debuffCount:Show();
				else
					debuffCount:Hide();
				end

				local isbuffadebuff = Perl_Player_BuffIsDebuff(buffIndex);
				if (isbuffadebuff == -1) then
					button.isdebuff = 0;
				else
					if (isbuffadebuff == -1 and buffIndex == 0) then
						debuffcheck[0] = 1;
					else
						debuffcheck[(isbuffadebuff + buffIndex)] = 1;
					end
					button.isdebuff = 1;
				end

				debuff:Hide();				-- Set this to hidden so the debuff loop below can properly set it
				timetext:SetTextColor(1, 1, 0, 1);	-- Set this to default color so the debuff loop below can properly set it

				if (untilCancelled == 0) then
					local timeleft = GetPlayerBuffTimeLeft(buffIndex);
					timetext:SetText(Perl_Player_GetStringTime(timeleft));
					timetext:Show();

					if (timeleft < BuffWarnTime) then
						local BuffAlpha;
						if (timeleft > 0) then
							BuffAlpha = BuffAlphaMin + 0.5 * (BuffAlphaMax - BuffAlphaMin) * (1 + math.sin(2 * math.pi * timeleft));
						else
							BuffAlpha = 0;
						end

						if (BuffAlpha > 1) then
							BuffAlpha = 1;
						elseif (BuffAlpha < 0) then
							BuffAlpha = 0;
						end
						button:SetAlpha(BuffAlpha);

						if ((button.isdebuff == 0) and (button.notwarned == 1) and (button.name == buffname) and (buffalerts == 1)) then
							UIErrorsFrame:AddMessage(buffname.." will expire in 30 seconds", 1.0, 1.0, 0.0, 1.0, UIERRORS_HOLD_TIME);
							DEFAULT_CHAT_FRAME:AddMessage("|cffffffff<"..buffname..">|cffffff00 will expire in 30 seconds");
							button.notwarned = 0;
						else
							button.notwarned = 0;
						end
					else
						button:SetAlpha(1);
						if (button.notwarned ~= 1) then
							button.notwarned = 1;
						end
					end
				else
					timetext:Hide();
					button:SetAlpha(1);
					button.notwarned = 0;
				end

				if ((buffname ~= nil) and (button.name ~= buffname)) then
					button.name = buffname;
					button.notwarned = 0;
				end
				button:Show();
			else
				button:Hide();
				button.notwarned = 0;
			end
		end

		for buffnum=0,24 do			-- Read the debuff table and set debuffs accordingly
			if (debuffcheck[buffnum] ~= 0) then
				local adjustedbuffnum = buffnum + 1;
				local debuff = getglobal("Perl_Player_Buff"..adjustedbuffnum.."DebuffBorder");
				local timetext = getglobal("Perl_Player_Buff"..adjustedbuffnum.."DurationText");

				debuff:Show();
				timetext:SetTextColor(1, 0, 0, 1);
			end
		end

		for buffnum=0,24 do			-- Reset the debuff table
			debuffcheck[buffnum] = 0;
		end
	else
		Perl_Player_BuffFrame:Hide();
	end

	if (not UIParent:IsShown()) then		-- Ugly way of making the buffs hide when ALT+Z is pressed, mod is still working when hidden
		Perl_Player_BuffFrame:Hide();
	else
		if (showbuffs == 1) then
			Perl_Player_BuffFrame:Show();
		else
			Perl_Player_BuffFrame:Hide();
		end
	end
end

function Perl_Player_BuffIsDebuff(index)
		local debuffIndex = GetPlayerBuff(index, "HARMFUL");
		if (debuffIndex == -1) then
			return -1;
		else
			return debuffIndex;
		end
end

function Perl_Player_GetStringTime(timenum)
	local minutes = math.floor(timenum / 60);
	local seconds = math.floor(timenum - 60 * minutes);
	local timestring;

	if (string.len(seconds) == 1) then
		seconds = "0"..seconds;
	end

	if (minutes > 60) then
		local hours = math.floor(minutes / 60);
		minutes = minutes - 60 * hours;
		if (string.len(minutes) == 1) then
			minutes = "0"..minutes;
		end
		timestring = hours..":"..minutes..":"..seconds;
	else
		timestring = minutes..":"..seconds;
	end

	return timestring;
end

function Perl_Player_SetBuffTooltip()
	local buffIndex = GetPlayerBuff((this:GetID() - 1), "HELPFUL|HARMFUL");

	GameTooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
	GameTooltip:SetPlayerBuff(buffIndex);
end

function Perl_Player_GetBuffName(buffIndex)
	if ((buffIndex < 24) and (buffIndex > -1)) then
		local tooltip = Perl_Player_Tooltip;
		if (tooltip) then
			tooltip:SetOwner(this, "ANCHOR_BOTTOMRIGHT");
			tooltip:SetPlayerBuff(buffIndex);
		end

		local toolTipText = getglobal("Perl_Player_TooltipTextLeft1");
		if (toolTipText) then
			local name = toolTipText:GetText();
			if (name ~= nil) then
				return name;
			end
		end
	end

	return nil;
end

function Perl_Player_BuffClicked(button)
	if (button == "RightButton") then
		local buffIndex = GetPlayerBuff((this:GetID() - 1), "HELPFUL|HARMFUL");
		CancelPlayerBuff(buffIndex);
		this.notwarned = 0;
	end
end

function Perl_Player_UseBuffs(useperlbuffs)
	if (useperlbuffs == 1) then
		BuffFrame:Hide();
	else
		BuffFrame:Show();
	end
end


--------------------------
-- GUI Config Functions --
--------------------------
function Perl_Player_Buff_Set_ShowBuffs(newvalue)
	showbuffs = newvalue;
	Perl_Player_Buff_UpdateVars();
	Perl_Player_UseBuffs(showbuffs);
end

function Perl_Player_Buff_Set_Alerts(newvalue)
	buffalerts = newvalue;
	Perl_Player_Buff_UpdateVars();
end

function Perl_Player_Buff_Set_Scale(number)
	local unsavedscale;
	if (number ~= nil) then
		scale = (number / 100);					-- convert the user input to a wow acceptable value
	end
	unsavedscale = 1 - UIParent:GetEffectiveScale() + scale;	-- run it through the scaling formula introduced in 1.9
	Perl_Player_BuffFrame:SetScale(unsavedscale);
	Perl_Player_Buff_UpdateVars();
end


------------------------------
-- Saved Variable Functions --
------------------------------
function Perl_Player_Buff_GetVars()
	buffalerts = Perl_Player_Buff_Config[UnitName("player")]["BuffAlerts"];
	showbuffs = Perl_Player_Buff_Config[UnitName("player")]["ShowBuffs"];
	scale = Perl_Player_Buff_Config[UnitName("player")]["Scale"];

	if (buffalerts == nil) then
		buffalerts = 1;
	end
	if (showbuffs == nil) then
		showbuffs = 1;
	end
	if (scale == nil) then
		scale = 1;
	end

	local vars = {
		["buffalerts"] = buffalerts,
		["showbuffs"] = showbuffs,
		["scale"] = scale,
	}
	return vars;
end

function Perl_Player_Buff_UpdateVars(vartable)
	if (vartable ~= nil) then
		-- Sanity checks in case you use a load from an old version
		if (vartable["Global Settings"] ~= nil) then
			if (vartable["Global Settings"]["BuffAlerts"] ~= nil) then
				buffalerts = vartable["Global Settings"]["BuffAlerts"];
			else
				buffalerts = nil;
			end
			if (vartable["Global Settings"]["ShowBuffs"] ~= nil) then
				showbuffs = vartable["Global Settings"]["ShowBuffs"];
			else
				showbuffs = nil;
			end
			if (vartable["Global Settings"]["Scale"] ~= nil) then
				scale = vartable["Global Settings"]["Scale"];
			else
				scale = nil;
			end
		end

		-- Set the new values if any new values were found, same defaults as above
		if (buffalerts == nil) then
			buffalerts = 1;
		end
		if (showbuffs == nil) then
			showbuffs = 1;
		end
		if (scale == nil) then
			scale = 1;
		end

		-- Call any code we need to activate them
		Perl_Player_UseBuffs(showbuffs);
		Perl_Player_Buff_Set_Scale();
	end

	Perl_Player_Buff_Config[UnitName("player")] = {
		["BuffAlerts"] = buffalerts,
		["ShowBuffs"] = showbuffs,
		["Scale"] = scale,
	};
end


----------------------
-- myAddOns Support --
----------------------
function Perl_Player_Buff_myAddOns_Support()
	-- Register the addon in myAddOns
	if(myAddOnsFrame_Register) then
		local Perl_Player_Buff_myAddOns_Details = {
			name = "Perl_Player_Buff",
			version = "11100.1",
			releaseDate = "June 20, 2006",
			author = "Perl; Maintained by Global",
			email = "global@g-ball.com",
			website = "http://www.curse-gaming.com/mod.php?addid=2257",
			category = MYADDONS_CATEGORY_OTHERS
		};
		Perl_Player_Buff_myAddOns_Help = {};
		Perl_Player_Buff_myAddOns_Help[1] = "/perlplayerbuff\n/ppb\n";
		myAddOnsFrame_Register(Perl_Player_Buff_myAddOns_Details, Perl_Player_Buff_myAddOns_Help);
	end
end